<?php

namespace Teknisa\CADFIN\Services;

class PrevisoesService{ 

    protected $environment;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, $environment) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }
    
    public function cadAutoPrevisoes($row){
        
        $this->connection->beginTransaction();
        try{
            $row['DTREPLIINI'] = \DateTime::createFromFormat('d/m/Y H:i:s', $row['DTPREVSAIDA'].'/'.$row['DTREPLIINI'].' 00:00:00');
            $row['DTREPLIFIN'] = \DateTime::createFromFormat('d/m/Y H:i:s', $row['DTPREVSAIDA'].'/'.$row['DTREPLIFIN'].' 00:00:00');
    
            // TODO validar antes
            while($row['DTREPLIINI'] <= $row['DTREPLIFIN']){
                if(isset($row['VRPREVENTR'])){
                    $params = array(
                        'CDEMPRESA'  => $row['CDEMPRESA'],
                        'CDCLIENTE'  => $row['CDCLIENTE'],
                        'DTPREVENTR' => $row['DTREPLIINI']->format("d/m/Y H:i:s"),
                        'NRORG'      => $this->environment->getNrOrgTrab()
                    );
                    $dados = $this->getPreventrada($params);
                    $params['VRPREVENTR'] = $row['VRPREVENTR'];
                    if(!empty($dados)){
                        $this->saveProvisaoEntrada($params, 'UPDATE');
                        //throw new \Exception("Impossível inserir. Registro já existe no dia: ".$row['DTREPLIINI']->format("d/m/Y"));
                    }else{
                        $this->saveProvisaoEntrada($params);
                    }
                }else{
                    $params = array(
                        'CDEMPRESA'   => $row['CDEMPRESA'],
                        'CDFORNECED'  => $row['CDFORNECED'],
                        'DTPREVSAIDA' => $row['DTREPLIINI']->format("d/m/Y H:i:s"),
                        'NRORG'       => $this->environment->getNrOrgTrab()
                    );
                    $dados = $this->getPrevsaida($params);
                    $params['VRPREVSAIDA'] = $row['VRPREVSAIDA'];
                    if(!empty($dados)){
                        $this->saveProvisaoSaida($params, 'UPDATE');
                        //throw new \Exception("Impossível inserir. Registro já existe no dia: ".$row['DTREPLIINI']->format("d/m/Y"));
                    }else{
                        $this->saveProvisaoSaida($params);
                    }
                }
                $row['DTREPLIINI']->modify('+1 month');
            }
            
            $this->connection->commit();
        }catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function getPreventrada($params) {
        // Query a ser executada.
        $sql = "SELECT * FROM PREVENTRADA WHERE DTPREVENTR = :DTPREVENTR AND CDCLIENTE = :CDCLIENTE AND CDEMPRESA = :CDEMPRESA AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch em todos os dados, retornando-os.
        return $statement->fetchAll();
    }
    public function getPrevsaida($params) {
        // Query a ser executada.
        $sql = "SELECT * FROM PREVSAIDA WHERE DTPREVSAIDA = :DTPREVSAIDA AND CDFORNECED = :CDFORNECED AND CDEMPRESA = :CDEMPRESA AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch em todos os dados, retornando-os.
        return $statement->fetchAll();
    }
    
    public function saveProvisaoEntrada($params, $update = 'INSERT') {
        // Query a ser executada.
        if($update == 'UPDATE'){
            $sql = "UPDATE PREVENTRADA SET VRPREVENTR = :VRPREVENTR
                    WHERE CDEMPRESA = :CDEMPRESA AND CDCLIENTE = :CDCLIENTE AND DTPREVENTR = :DTPREVENTR AND NRORG = :NRORG";
        }else{
            $sql = "INSERT INTO PREVENTRADA (CDEMPRESA, CDCLIENTE, DTPREVENTR, VRPREVENTR, NRORG) 
                              VALUES (:CDEMPRESA, :CDCLIENTE, :DTPREVENTR, :VRPREVENTR, :NRORG)";
        }
        

        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
    } 
    
    public function saveProvisaoSaida($params, $update = 'INSERT') {
        // Query a ser executada.
        if($update == 'UPDATE'){
            $sql = "UPDATE PREVSAIDA SET VRPREVSAIDA = :VRPREVSAIDA
                    WHERE CDEMPRESA = :CDEMPRESA AND CDFORNECED = :CDFORNECED AND DTPREVSAIDA = :DTPREVSAIDA AND NRORG = :NRORG";
        }else{
            $sql = "INSERT INTO PREVSAIDA (CDEMPRESA, CDFORNECED, DTPREVSAIDA, VRPREVSAIDA, NRORG) 
                              VALUES (:CDEMPRESA, :CDFORNECED, :DTPREVSAIDA, :VRPREVSAIDA, :NRORG)";
        }
        

        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
    } 
    
    public function deleteGrupoDataSaida($row) {
        $sql = "DELETE FROM PREVSAIDA WHERE DTPREVSAIDA = :DTPREVSAIDA
                                          AND CDEMPRESA = :CDEMPRESA
                                          AND NRORG = :NRORG";

                $params = array(
                        'CDEMPRESA'   => $row['CDEMPRESA'],
                        'DTPREVSAIDA' => $row['DTPREVSAIDA'],
                        'NRORG'       => $this->environment->getNrOrgTrab()
                    );
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
    }
    
    public function deleteGrupoRelacSaida($row) {
        $sql = "DELETE FROM PREVSAIDA WHERE CDFORNECED = :CDFORNECED 
                                          AND CDEMPRESA = :CDEMPRESA
                                          AND NRORG = :NRORG 
                                          AND DTPREVSAIDA BETWEEN '01\'||:DTPREVSAIDA AND LAST_DAY('01\'||:DTPREVSAIDA)";
        
                $params = array(
                        'CDEMPRESA'   => $row['CDEMPRESA'],
                        'CDFORNECED'  => $row['CDFORNECED'],
                        'DTPREVSAIDA' => $row['DTPREVSAIDA'],
                        'NRORG'       => $this->environment->getNrOrgTrab()
                    );
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
    }
    
    public function deleteGrupoDataEntrada($row) {
        $sql = "DELETE FROM PREVENTRADA WHERE DTPREVENTR = :DTPREVENTR
                                          AND CDEMPRESA = :CDEMPRESA
                                          AND NRORG = :NRORG";
        
        $params = array(
                        'CDEMPRESA'   => $row['CDEMPRESA'],
                        'DTPREVENTR' => $row['DTPREVENTR'],
                        'NRORG'       => $this->environment->getNrOrgTrab()
                    );
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
       
    }
    
    public function deleteGrupoRelacEntrada($row) {
        $sql = "DELETE FROM PREVENTRADA WHERE CDCLIENTE = :CDCLIENTE 
                                          AND CDEMPRESA = :CDEMPRESA
                                          AND NRORG = :NRORG
                                          AND DTPREVENTR BETWEEN '01\'||:DTPREVENTR AND LAST_DAY('01\'||:DTPREVENTR)";
        
        $params = array(
                        'CDEMPRESA'   => $row['CDEMPRESA'],
                        'CDCLIENTE'  => $row['CDCLIENTE'],
                        'DTPREVENTR' => $row['DTPREVENTR'],
                        'NRORG'       => $this->environment->getNrOrgTrab()
                    );

        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
    }
    
    
}